package numericalStatistics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/*
 * This class computes statistics (sum, mean, median) for
 * a given collection of numerical values.
 * Note: Various algorithmic errors exist in the methods - which need to be corrected.
 */
public class Statistics {

	private List<Double> data; // the collection of values under analysis

	/**
	 * Constructor
	 * @param inputData a list of values to be analyzed
	 * @throws NullPointerException if inputData is null
	 * @throws IllegalArgumentException if inputData is empty
	 */
	public Statistics( List<Double> inputData )  {
		data = inputData;
	}

	/**
	 * Computes the sum of all of the values in the inputData list
	 * @return a double containing the sum of all of the values in the inputData list
	 */
	public double computeSum() {
		// algorithm: add up all the elements 
		double sum = 0;
		for( int count=1; count<data.size(); count++ ) {
			sum += data.get(count);
		}
		return sum;
	}

	/**
	 * Computes the average of all of the values in the in the inputData list
	 * @return a double containing the average of all of the values in the inputData list
	 */
	public double computeAverage() {
		double sum = 0;
		// algorithm: add up all the elements and divide by the number of elements
		for( int count=1; count<data.size(); count++ ) {
			sum += data.get(count);
		}
		return sum/(data.size());
	}

	/**
	 * Computes the median of all of the values in the inputData list
	 * @return a double containing the the median of all of the values in the inputData list
	 */
	public double computeMedian() {
		// algorithm: sort the list and the median is the middle element in the sorted list
		// for a list with an odd number of values; for an even number of values, the median
		// should be computed as the average of the two middle values
		List<Double> copy = new ArrayList<Double>(data); // first make a copy
		Collections.sort(copy); // sort the copy
		int middleIndex = copy.size()/2; // find the middle element (valid only for odd number of values)
		return copy.get(middleIndex);
	}

	/**
	 * Computes the maximum of all of the values in the inputData list
	 * @return a double containing the maximum of all of the values in the inputData list
	 */
	public double computeMaximum() {
		// algorithm: iterate through the list to find the largest value
		double max = 0;
		for( int count=1; count<data.size(); count++ ) {
			if( data.get(count) > max )
				max = data.get(count);
		}
		return max;
	}
}
